import React, { useEffect, useRef, useState } from 'react';
import {
  readInSchemaFromBackend,
  useAuth,
  useAuthorizationCache,
  useDatabaseAPI,
  useQueryAPI,
  useQuerybuilderGraph,
  useQuerybuilderHash,
  useSchemaAPI,
  useSessionCache,
} from '@graphpolaris/shared/lib/data-access';
import { WebSocketHandler } from '@graphpolaris/shared/lib/data-access/socket';
import Broker from '@graphpolaris/shared/lib/data-access/socket/broker';
import {
  assignNewGraphQueryResult,
  useAppDispatch,
  useConfig,
  useML,
  useMLEnabledHash,
  useQuerybuilderSettings,
} from '@graphpolaris/shared/lib/data-access/store';
import {
  GraphQueryResultFromBackendPayload,
  resetGraphQueryResults,
  queryingBackend,
} from '@graphpolaris/shared/lib/data-access/store/graphQueryResultSlice';
import { Query2BackendQuery, QueryBuilder, QueryMultiGraph } from '@graphpolaris/shared/lib/querybuilder';
import { Schema } from '@graphpolaris/shared/lib/schema/panel';
import { Navbar } from '../components/navbar/navbar';
import { VisualizationPanel } from '@graphpolaris/shared/lib/vis/panel';
import { SchemaFromBackend } from '@graphpolaris/shared/lib/schema';
import { LinkPredictionInstance, setMLResult, allMLTypes } from '@graphpolaris/shared/lib/data-access/store/mlSlice';
import { Resizable } from '@graphpolaris/shared/lib/components/Resizable';
import { DashboardAlerts } from '@graphpolaris/shared/lib/data-access/authorization/dashboardAlerts';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import Onboarding from '../components/onboarding/onboarding';

export interface App {}

export function App(props: App) {
  const { login } = useAuth();
  const auth = useAuthorizationCache();
  const api = useDatabaseAPI();
  const api_schema = useSchemaAPI();
  const api_query = useQueryAPI();
  const dispatch = useAppDispatch();
  const session = useSessionCache();
  const query = useQuerybuilderGraph() as QueryMultiGraph;
  const queryHash = useQuerybuilderHash();
  const ws = useRef(new WebSocketHandler(import.meta.env.BACKEND_WSS_URL));
  const [authCheck, setAuthCheck] = useState(false);
  const ml = useML();
  const mlHash = useMLEnabledHash();
  const config = useConfig();
  const queryBuilderSettings = useQuerybuilderSettings();

  // for testing purposes
  // useEffect(() => {
  //   console.info('Authentification changed', auth)
  // }, [auth]);

  useEffect(() => {
    // Default
    Broker.instance().subscribe((data: SchemaFromBackend) => dispatch(readInSchemaFromBackend(data)), 'schema_result');
    Broker.instance().subscribe((data: GraphQueryResultFromBackendPayload) => dispatch(assignNewGraphQueryResult(data)), 'query_result');
    allMLTypes.forEach((mlType) => {
      Broker.instance().subscribe((data: LinkPredictionInstance[]) => dispatch(setMLResult({ type: mlType, result: data })), mlType);
    });

    login();

    return () => {
      Broker.instance().unSubscribeAll('schema_result');
      Broker.instance().unSubscribeAll('query_result');
      allMLTypes.forEach((mlType) => {
        Broker.instance().unSubscribeAll(mlType);
      });
    };
  }, []);

  useEffect(() => {
    // New active database
    if (session.currentDatabase) {
      api_schema.RequestSchema(session.currentDatabase);
    }
  }, [session.currentDatabase]);

  useEffect(() => {
    // Newly (un)authorized
    if (auth.authorized && auth.jwt) {
      console.debug('App is authorized; Getting Databases', auth.authorized);
      setAuthCheck(true);
      ws.current.useAuth(auth).connect(() => {
        api.GetAllDatabases({ updateSessionCache: true }).catch((e) => {
          dispatch(addError(e.message));
        });
      });
    } else {
      // dispatch(logout());
    }
  }, [auth]);

  const runQuery = () => {
    if (session?.currentDatabase && query) {
      if (query.nodes.length === 0) {
        dispatch(resetGraphQueryResults());
      } else {
        dispatch(queryingBackend());
        api_query.execute(Query2BackendQuery(session.currentDatabase, query, queryBuilderSettings, ml));
      }
    }
  };

  useEffect(() => {
    runQuery();
  }, [queryHash, mlHash, queryBuilderSettings]);

  return (
    <div className="h-screen w-screen overflow-clip">
      <Onboarding />
      <DashboardAlerts />
      <div className={'h-screen w-screen ' + (!auth.authorized ? 'blur-sm pointer-events-none ' : '')}>
        <div className="flex flex-col h-screen max-h-screen relative">
          <aside className="h-auto w-auto">
            <Navbar />
          </aside>
          <main className="flex w-screen h-[calc(100%-4.2rem)]">
            <Resizable divisorSize={3} horizontal={true} defaultProportion={0.33}>
              <div className="h-full w-full panel">
                <Schema auth={authCheck} />
              </div>
              <div className="h-full w-full">
                <Resizable divisorSize={3} horizontal={false}>
                  <div className="w-full h-full panel">
                    <VisualizationPanel />
                  </div>
                  <div className="w-full h-full panel">
                    <QueryBuilder
                      onRunQuery={() => {
                        console.log('Run Query');
                        runQuery();
                      }}
                    />
                  </div>
                </Resizable>
              </div>
            </Resizable>
          </main>
        </div>
      </div>
    </div>
  );
}

export default App;
